package com.fireworksdk.bridge.components.storyblock

import android.graphics.Color
import android.graphics.Outline
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.LayoutInflater
import android.view.SurfaceView
import android.view.View
import android.view.View.LAYOUT_DIRECTION_LOCALE
import android.view.ViewGroup
import android.view.ViewGroup.LayoutParams
import android.view.ViewOutlineProvider
import android.widget.FrameLayout
import com.firework.error.FwErrorListener
import com.firework.storyblock.FeedLoadListener
import com.firework.storyblock.FeedLoadState
import com.firework.storyblock.FwStoryBlockView
import com.fireworksdk.bridge.components.base.FWBaseFragment
import com.fireworksdk.bridge.components.base.FWLoading
import com.fireworksdk.bridge.databinding.FwBridgeStoryBlockBinding
import com.fireworksdk.bridge.models.FWVideoFeedPropsModel
import com.fireworksdk.bridge.models.FWVideoFeedPropsModelDeserializer
import com.fireworksdk.bridge.models.FWVideoFeedPropsModelSerializer
import com.fireworksdk.bridge.utils.FWCommonUtil
import com.fireworksdk.bridge.utils.FWConfigUtil
import com.fireworksdk.bridge.utils.FWGlobalDataUtil
import com.fireworksdk.bridge.utils.FWLanguageUtil
import org.json.JSONObject
import java.lang.reflect.Method

class StoryBlockFragment : FWBaseFragment() {

  private var _binding: FwBridgeStoryBlockBinding? = null
  private val binding: FwBridgeStoryBlockBinding get() = _binding!!

  private var feedLoadListener: FeedLoadListener? = null
  private var errorListener: FwErrorListener? = null
  private var videoFeedPropsModel: FWVideoFeedPropsModel? = null

  private var isStoryBlockInitializer = false
  private var storyBlockLayoutParams: FrameLayout.LayoutParams? = null

  private var fwStoryBlockView: FwStoryBlockView? = null

  private val runnable = Runnable {
    removeLoadingViewFromIvsPlayerView(fwStoryBlockView)
  }
  private var hasAdded = false

  private var loaderContainer: FrameLayout? = null

  private var needIvsLoading = false

  override fun onCreateView(
    inflater: LayoutInflater,
    container: ViewGroup?,
    savedInstanceState: Bundle?,
  ): View {
    _binding = FwBridgeStoryBlockBinding.inflate(inflater)

    val videoFeedPropsModelJsonString  = arguments?.getString(VIDEO_FEED_PROPS_MODEL)
    videoFeedPropsModelJsonString?.let {
      val videoFeedPropsModelJsonObject = JSONObject(it)
      val model = FWVideoFeedPropsModelDeserializer.deserialize(videoFeedPropsModelJsonObject)
      videoFeedPropsModel = model
    }
    val needIvsLoading = arguments?.getBoolean(NEED_IVS_LOADING) ?: false
    this.needIvsLoading = needIvsLoading

    loaderContainer = binding.fwLoadingWrapper.loaderContainer

    val right = videoFeedPropsModel?.marginRight
    if (right != null && right > 0) {
      val layoutParams = FrameLayout.LayoutParams(
        LayoutParams.MATCH_PARENT,
        LayoutParams.MATCH_PARENT
      )
      layoutParams.setMargins(0, 0, FWCommonUtil.dpToPx(right.toFloat(), requireContext()), 0)
      binding.storyBlockContainer.layoutParams = layoutParams

      if (needIvsLoading) {
        loaderContainer?.visibility = View.VISIBLE
        loaderContainer?.layoutParams = layoutParams
        loaderContainer?.setBackgroundColor(Color.BLACK)
      }
    }

    FWLanguageUtil.getInstance(requireContext()).updateBaseContextLocale(requireContext())
    binding.storyBlock.layoutDirection = LAYOUT_DIRECTION_LOCALE

    fwStoryBlockView = binding.storyBlock
    return binding.root
  }

  override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
    super.onViewCreated(view, savedInstanceState)
    setStoryBlockContainerCorner()
    initStoryBlock()
    createViewCallback?.onCreateView()
  }

  fun getFeedId(): String? {
    return fwStoryBlockView?.feedId
  }


  override fun onDestroyView() {
    fwStoryBlockView?.destroy()
    _binding = null
    super.onDestroyView()
  }

  private fun initStoryBlock() {
    val viewOptionsBuilder = FWConfigUtil.generateViewOptionsBuilder(context, videoFeedPropsModel)

    fwStoryBlockView?.init(
      childFragmentManager,
      viewLifecycleOwner,
      viewOptionsBuilder.build(),
      FWGlobalDataUtil.pauseWhenNotVisible
    )
    isStoryBlockInitializer = true

    if (storyBlockLayoutParams != null) {
      binding.storyBlockContainer.layoutParams = storyBlockLayoutParams
      binding.root.requestLayout()
    }
    // must call it after init()
    setFeedLoadListener(feedLoadListener)
    setOnErrorListener(errorListener)
  }

  // may be called before onCreateView
  fun setStoryBlockLayoutParams(l: FrameLayout.LayoutParams?) {
    l?: return
    storyBlockLayoutParams = l
    if (isStoryBlockInitializer) {
      binding.storyBlockContainer.layoutParams = l
      binding.root.requestLayout()
    }
  }

  // may be called before onCreateView
  fun setFeedLoadListener(feedLoadListener: FeedLoadListener?) {
    this.feedLoadListener = feedLoadListener
    if (isStoryBlockInitializer) {
      fwStoryBlockView?.setFeedLoadListener { feedLoadState ->
        feedLoadListener?.onFeedLoadStateChanged(feedLoadState)
        if (feedLoadState !== FeedLoadState.Loading) {
          hideLoading()
        }
      }
    }
  }

  fun setOnErrorListener(errorListener: FwErrorListener?) {
    errorListener ?: return
    this.errorListener = errorListener
    if (isStoryBlockInitializer) {
      fwStoryBlockView?.setOnErrorListener { fwError ->
        hideLoading()
        errorListener.onFwError(fwError)
      }
    }
  }

  fun play() {
    fwStoryBlockView?.play()
  }

  fun pause() {
    fwStoryBlockView?.pause()
  }

  fun onViewportEntered() {
    fwStoryBlockView?.onViewportEntered()
    if (hasAdded && needIvsLoading) {
      hasAdded = false
      removeLoadingViewFromIvsPlayerView()
    }
  }

  fun onViewportLeft() {
    fwStoryBlockView?.onViewportLeft()
    if (!hasAdded && needIvsLoading) {
      hasAdded = true
      addLoadingViewToIvsPlayerView()
    }
  }

  private fun addLoadingViewToIvsPlayerView() {
    Handler(Looper.getMainLooper()).removeCallbacks(runnable)

    val ivsPlayerViewList = mutableListOf<ViewGroup>()
    searchVisibleIvsPlayerView(fwStoryBlockView, ivsPlayerViewList)
    for (view in ivsPlayerViewList) {
      val loading = FWLoading(view.context)
      loading.setBackgroundColor(Color.BLACK)
      view.addView(loading, LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT))
    }
  }

  private fun removeLoadingViewFromIvsPlayerView() {
    val storyBlockView  = fwStoryBlockView
    storyBlockView ?: return

    Handler(Looper.getMainLooper()).removeCallbacks(runnable)
    Handler(Looper.getMainLooper()).postDelayed(runnable, 1000)
  }


  fun hideIvsPlayerView() {
    val ivsPlayerViewList = mutableListOf<ViewGroup>()
    searchVisibleIvsPlayerView(fwStoryBlockView, ivsPlayerViewList)
    hideIvsPlayerSurfaceView(ivsPlayerViewList)
  }

  fun showIvsPlayerView() {
    val ivsPlayerViewList = mutableListOf<ViewGroup>()
    searchVisibleIvsPlayerView(fwStoryBlockView, ivsPlayerViewList)
    showIvsPlayerSurfaceView(ivsPlayerViewList)
  }

  fun removeIvsPlayerView() {
    val ivsPlayerViewList = mutableListOf<ViewGroup>()
    searchVisibleIvsPlayerView(fwStoryBlockView, ivsPlayerViewList)
    removeIvsPlayerSurfaceView(ivsPlayerViewList)
  }

  fun playIvsPlayer() {
    val ivsPlayerViewList = mutableListOf<ViewGroup>()
    searchVisibleIvsPlayerView(fwStoryBlockView, ivsPlayerViewList)
    playIvsPlayerView(ivsPlayerViewList)
  }

  fun pauseIvsPlayer() {
    val ivsPlayerViewList = mutableListOf<ViewGroup>()
    searchVisibleIvsPlayerView(fwStoryBlockView, ivsPlayerViewList)
    pauseIvsPlayerView(ivsPlayerViewList)
  }

  private fun searchVisibleIvsPlayerView(viewGroup: ViewGroup?, list: MutableList<ViewGroup>) {
    viewGroup ?: return

    val childCount = viewGroup.childCount
    for (index in 0 until childCount) {
      val view = viewGroup.getChildAt(index)
      if (view is ViewGroup) {
        val className = view.javaClass.name
        if (className.equals("com.amazonaws.ivs.player.PlayerView")
          || className.equals("com.amazonaws.ivs.player.CustomPlayerView")) {
          list.add(view)
        } else {
          searchVisibleIvsPlayerView(view, list)
        }
      }
    }
  }

  private fun hideIvsPlayerSurfaceView(list: List<ViewGroup>?) {
    list ?: return
    for (viewGroup in list) {
      val childCount = viewGroup.childCount
      for (index in 0 until childCount) {
        val view = viewGroup.getChildAt(index)
        if (view is SurfaceView) {
          view.visibility = View.INVISIBLE
        }
      }
    }
  }

  private fun showIvsPlayerSurfaceView(list: List<ViewGroup>?) {
    list ?: return
    for (viewGroup in list) {
      val childCount = viewGroup.childCount
      for (index in 0 until childCount) {
        val view = viewGroup.getChildAt(index)
        if (view is SurfaceView) {
          view.visibility = View.VISIBLE
        }
      }
    }
  }

  private fun removeIvsPlayerSurfaceView(list: List<ViewGroup>?) {
    list ?: return
    for (viewGroup in list) {
      val childCount = viewGroup.childCount
      for (index in 0 until childCount) {
        val view = viewGroup.getChildAt(index)
        if (view is SurfaceView) {
          viewGroup.removeView(view)
        }
      }
    }
  }

  fun openFullscreen() {
    fwStoryBlockView?.openFullscreen()
  }

  companion object {
    private const val VIDEO_FEED_PROPS_MODEL = "videoFeedPropsModel"
    private const val NEED_IVS_LOADING = "needIvsLoading"

    fun newInstance(
      videoFeedPropsModel: FWVideoFeedPropsModel?,
      needIvsLoading: Boolean? = false,
    ): StoryBlockFragment {
      return StoryBlockFragment().apply {
        arguments = Bundle().apply {
          videoFeedPropsModel?.let {
            putString(VIDEO_FEED_PROPS_MODEL, FWVideoFeedPropsModelSerializer.serialize(it).toString())
          }
          needIvsLoading?.let {
            putBoolean(NEED_IVS_LOADING, needIvsLoading)
          }
        }
      }
    }
  }

  private fun setStoryBlockContainerCorner() {
    val cornerRadius = videoFeedPropsModel?.cornerRadius
    if (cornerRadius != null && cornerRadius >= 0) {
      val cornerRadiusPx = FWCommonUtil.dpToPx(cornerRadius.toFloat(), requireContext()).toFloat()

      val outlineProvider = object : ViewOutlineProvider() {
        override fun getOutline(view: View?, outline: Outline?) {
          view?.let {
            outline?.setRoundRect(0, 0, view.width, view.height, cornerRadiusPx)
          }
        }
      }
      binding.storyBlockContainer.outlineProvider = outlineProvider
      binding.storyBlockContainer.clipToOutline = true

      loaderContainer?.outlineProvider = outlineProvider
      loaderContainer?.clipToOutline = true
    }
  }

  private fun pauseIvsPlayerView(list: List<ViewGroup>?) {
    list ?: return
    for (view in list) {
      if (view.javaClass.name.equals("com.amazonaws.ivs.player.CustomPlayerView")) {
        val filed = view.javaClass.getDeclaredField("player")
        filed.isAccessible = true
        val player = filed.get(view)
        val pauseMethod: Method = player.javaClass.getDeclaredMethod("pause")
        pauseMethod.invoke(player)
      } else if (view.javaClass.name.equals("com.amazonaws.ivs.player.PlayerView")) {
        val getPlayerMethod: Method = view.javaClass.getDeclaredMethod("getPlayer")
        val player = getPlayerMethod.invoke(view)
        val pauseMethod: Method = player.javaClass.getDeclaredMethod("pause")
        pauseMethod.invoke(player)
      }
    }
  }

  private fun playIvsPlayerView(list: List<ViewGroup>?) {
    list ?: return
    for (view in list) {
      if (view.javaClass.name.equals("com.amazonaws.ivs.player.CustomPlayerView")) {
        val filed = view.javaClass.getDeclaredField("player")
        filed.isAccessible = true
        val player = filed.get(view)
        val pauseMethod: Method = player.javaClass.getDeclaredMethod("play")
        pauseMethod.invoke(player)
      } else if (view.javaClass.name.equals("com.amazonaws.ivs.player.PlayerView")) {
        val getPlayerMethod: Method = view.javaClass.getDeclaredMethod("getPlayer")
        val player = getPlayerMethod.invoke(view)
        val playMethod: Method = player.javaClass.getDeclaredMethod("play")
        playMethod.invoke(player)
      }
    }
  }

  private fun removeLoadingViewFromIvsPlayerView(viewGroup: ViewGroup?) {
    viewGroup ?: return

    val childCount = viewGroup.childCount
    for (index in 0 until childCount) {
      val view = viewGroup.getChildAt(index)
      if (view is ViewGroup) {
        val className = view.javaClass.name
        if (className.equals("com.fireworksdk.bridge.components.base.FWLoading")) {
          viewGroup.removeView(view)
        } else {
          removeLoadingViewFromIvsPlayerView(view)
        }
      }
    }
  }

  private fun hideLoading() {
    if (loaderContainer?.visibility == View.VISIBLE) {
      Handler(Looper.getMainLooper()).postDelayed({
        loaderContainer?.visibility = View.GONE
      }, 1000)
    }
  }

  private var createViewCallback: OnCreateViewCallback? = null

  fun setCreateViewCallback(createViewCallback: OnCreateViewCallback?) {
    this.createViewCallback = createViewCallback
  }

  interface OnCreateViewCallback {
    fun onCreateView()
  }

}
