package com.fireworksdk.bridge.flutter.handler

import android.content.Context
import androidx.annotation.Keep
import com.fireworksdk.bridge.flutter.models.FWSdkModuleCallMethod
import com.fireworksdk.bridge.models.*
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import kotlin.collections.HashMap
import com.firework.analyticsevents.cta.CtaButtonClickAnalyticsEvent
import com.firework.analyticsevents.player.PlayerLifecycleAnalyticsEvent
import com.firework.analyticsevents.share.ShareButtonAnalyticsEvent
import com.firework.bus.FwAnalyticCallable
import com.firework.sdk.FireworkSdk
import com.fireworksdk.bridge.FWInitializationProvider
import com.fireworksdk.bridge.flutter.FWFlutterSDK
import com.fireworksdk.bridge.flutter.utils.FWEventUtils
import com.fireworksdk.bridge.models.enums.FWVideoPlaybackSubEventName
import com.fireworksdk.bridge.utils.FWConfigUtil
import com.fireworksdk.bridge.utils.FWGlobalDataUtil
import com.fireworksdk.bridge.utils.FWLanguageUtil
import com.fireworksdk.bridge.utils.FWModelUtils
import org.json.JSONObject


class FWSdkModuleHandler(
  private val context: Context,
  channel: MethodChannel?
) : MethodCallHandler {

  private val methodChannel by weakProperty<MethodChannel?>(channel)

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {
      FWSdkModuleCallMethod.Init.rawValue -> {
        val configMap = call.arguments
        val jsonObject = JSONObject(configMap as HashMap<*, *>)
        val sdkInitOptionsModel = FWSDKInitOptionsModelDeserializer.deserialize(jsonObject)
        initCalled(sdkInitOptionsModel)
        FireworkSdk.analytics.register(this)

        result.success(null)
      }

      FWSdkModuleCallMethod.MarkInitCalled.rawValue -> {
        val sdkInitResultModel = FWGlobalDataUtil.sdkInitResultModel
        if (sdkInitResultModel != null) { // init in application
          if (sdkInitResultModel.success == true) {
            FWEventUtils.sendInitSuccessEvent(methodChannel)
          } else if (sdkInitResultModel.success == false) {
            FWEventUtils.sendInitFailedEvent(methodChannel, sdkInitResultModel.reason)
          }

          FireworkSdk.analytics.register(this)

          result.success(true)
          return
        }

        result.success(false)
      }

      FWSdkModuleCallMethod.OpenVideoPlayer.rawValue -> {
        val map = call.arguments as HashMap<*, *>
        if (map.size > 0) {
          val url = map["url"] as String
          val configMap = map["config"]
          val jsonObject = JSONObject(configMap as HashMap<*, *>)
          val videoPlayerConfigModel = FWVideoPlayerConfigModelDeserializer.deserialize(jsonObject)

          val videoFeedPropsModel = FWVideoFeedPropsModel(
            videoPlayerConfiguration = videoPlayerConfigModel
          )
          val viewOptionsBuilder = FWConfigUtil.generateViewOptionsBuilder(context, videoFeedPropsModel)
          FireworkSdk.startPlayer(viewOptionsBuilder.build(), url)
        }
        result.success(null)
      }

      FWSdkModuleCallMethod.SetShareBaseURL.rawValue -> {
        val url: String? = call.arguments as String?
        url?.let {
          FWGlobalDataUtil.shareBaseUrl = it
        }
        result.success(null)
      }

      FWSdkModuleCallMethod.SetAdBadgeConfiguration.rawValue -> {
        val configMap = call.arguments
        val jsonObject = JSONObject(configMap as HashMap<*, *>)
        val adBadgeConfigModel = FWAdBadgeConfigModelDeserializer.deserialize(jsonObject)
        FWGlobalDataUtil.adBadgeConfigModel = adBadgeConfigModel ?: FWAdBadgeConfigModel()
        result.success(null)
      }

      FWSdkModuleCallMethod.SetCustomCTAClickEnabled.rawValue -> {
        val value: Boolean? = call.arguments as Boolean?
        FWGlobalDataUtil.customCTAClickEnabled = value?:false
        result.success(null)
      }

      FWSdkModuleCallMethod.SetVideoPlaybackEventEnabled.rawValue -> {
        val value: Boolean? = call.arguments as Boolean?
        FWGlobalDataUtil.videoPlaybackEventEnabled = value?:false
        result.success(null)
      }

      FWSdkModuleCallMethod.ChangeAppLanguage.rawValue -> {
        val activity = FWInitializationProvider.INSTANCE.resumedActivity
        if (activity == null) {
          result.success(false)
          return
        }
        val language: String? = call.arguments as String?
        val changeLanguageResult = FWLanguageUtil.getInstance(activity).changeLanguage(language, activity)
        result.success(changeLanguageResult)
      }

      FWSdkModuleCallMethod.TrackPurchase.rawValue -> {
        val map = call.arguments as HashMap<*, *>
        val jsonObject = JSONObject(map)
        val trackPurchaseModel = FWTrackPurchaseModelDeserializer.deserialize(jsonObject)

        if (trackPurchaseModel?.orderId == null) {
          result.success(false)
          return
        }
        FireworkSdk.shopping.trackPurchase(
          trackPurchaseModel.orderId,
          trackPurchaseModel.value,
          trackPurchaseModel.currencyCode,
          trackPurchaseModel.countryCode,
          trackPurchaseModel.additionalInfo ?: mutableMapOf()
        )
        result.success(true)
      }

      else -> {
        result.notImplemented()
      }
    }
  }

  @Keep
  @FwAnalyticCallable
  fun onCtaButtonClicked(event: CtaButtonClickAnalyticsEvent) {
    val videoPlaybackDetails = FWModelUtils.convertCtaEvent2PlaybackDetails(event)

    if (FWGlobalDataUtil.videoPlaybackEventEnabled) {
      FWEventUtils.sendVideoPlaybackEvent(
        methodChannel,
        FWVideoPlaybackSubEventName.ClickCTA.rawValue,
        videoPlaybackDetails
      )
    }

    FWEventUtils.sendCustomCTAClickEvent(methodChannel, event.actionUrl, videoPlaybackDetails)
  }

  @Keep
  @FwAnalyticCallable
  fun onShareButtonClicked(event: ShareButtonAnalyticsEvent) {
    if (FWGlobalDataUtil.videoPlaybackEventEnabled) {
      FWEventUtils.sendVideoPlaybackEvent(
        methodChannel,
        FWVideoPlaybackSubEventName.ClickShare.rawValue,
        FWModelUtils.convertShareButtonEvent2PlaybackDetails(event)
      )
    }
  }

  @Keep
  @FwAnalyticCallable
  fun onPlaybackEvent(event: PlayerLifecycleAnalyticsEvent) {
    if (!FWGlobalDataUtil.videoPlaybackEventEnabled) {
      return
    }
    val videoPlaybackDetails = FWModelUtils.convertPlayerEvent2PlaybackDetails(event)
    when (event) {
      is PlayerLifecycleAnalyticsEvent.OnStarted -> {
        if (event.videoInfo?.isAd == true) {
          FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.AdStart.rawValue, videoPlaybackDetails)
        } else {
          FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.Start.rawValue, videoPlaybackDetails)
        }
      }
      is PlayerLifecycleAnalyticsEvent.OnPrepared -> {
        FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.Impression.rawValue, videoPlaybackDetails)
      }
      is PlayerLifecycleAnalyticsEvent.OnBuffering -> {}
      is PlayerLifecycleAnalyticsEvent.OnIdle -> {}
      is PlayerLifecycleAnalyticsEvent.OnRepeat -> {}
      is PlayerLifecycleAnalyticsEvent.OnEnded -> {
        if (event.videoInfo?.isAd == true) {
          FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.AdEnd.rawValue, videoPlaybackDetails)
        } else {
          FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.Complete.rawValue, videoPlaybackDetails)
        }
      }
      is PlayerLifecycleAnalyticsEvent.OnError -> {}
      is PlayerLifecycleAnalyticsEvent.OnSeek -> {}
      is PlayerLifecycleAnalyticsEvent.OnVideoSizeChanged -> {}
      is PlayerLifecycleAnalyticsEvent.OnFirstQuartile -> {
        FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.FirstQuartile.rawValue, videoPlaybackDetails)
      }
      is PlayerLifecycleAnalyticsEvent.OnSecondQuartile -> {
        FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.Midpoint.rawValue, videoPlaybackDetails)
      }
      is PlayerLifecycleAnalyticsEvent.OnThirdQuartile -> {
        FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.ThirdQuartile.rawValue, videoPlaybackDetails)
      }
      is PlayerLifecycleAnalyticsEvent.OnPlaybackProgressChanged -> {}
      is PlayerLifecycleAnalyticsEvent.OnPaused -> {
        FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.Pause.rawValue, videoPlaybackDetails)
      }
      is PlayerLifecycleAnalyticsEvent.OnResumed -> {
        FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.Resume.rawValue, videoPlaybackDetails)
      }
      else -> {}
    }
  }

  private fun initCalled(sdkInitOptionsModel: FWSDKInitOptionsModel? = null) {
    val sdkInitResultModel = FWGlobalDataUtil.sdkInitResultModel
    if (sdkInitResultModel == null) {
      // not init in application
      FWFlutterSDK.initSdkInternal(context, sdkInitOptionsModel, onSuccess = {
        FWEventUtils.sendInitSuccessEvent(methodChannel)
      }, onError = {
        FWEventUtils.sendInitFailedEvent(methodChannel, it)
      })
    } else {
      // init in application
      if (sdkInitResultModel.success == true) {
        FWEventUtils.sendInitSuccessEvent(methodChannel)
      } else if (sdkInitResultModel.success == false) {
        FWEventUtils.sendInitFailedEvent(methodChannel, sdkInitResultModel.reason)
      }
    }
  }
}