package com.fireworksdk.bridge.utils

import android.app.Activity
import android.content.Context
import androidx.fragment.app.FragmentActivity
import com.fireworksdk.bridge.components.base.FWBaseFragment
import java.util.Locale

class FWLanguageUtil private constructor(context: Context) {

  private val sharedPreferences by lazy {
    context.getSharedPreferences(LOCALE_SP, Context.MODE_PRIVATE)
  }

  private val initAppLocale: Locale = Locale.getDefault()

  private var localeString: String? = null

  init {
    localeString = sharedPreferences.getString(LOCALE_KEY, null)
  }

  fun updateBaseContextLocale(baseContext: Context): Context {
    val currentLocaleString = localeString

    if (currentLocaleString.isNullOrBlank()) {
      Locale.setDefault(initAppLocale)
      return updateResourcesLocale(baseContext, initAppLocale)
    }

    val localeStrings = currentLocaleString.split("-")
    val locale = if (localeStrings.size > 1) {
      Locale(localeStrings[0], localeStrings[1])
    } else {
      Locale(currentLocaleString)
    }
    Locale.setDefault(locale)
    return updateResourcesLocale(baseContext, locale)
  }

  private fun updateResourcesLocale(context: Context, locale: Locale): Context {
    val resources = context.resources
    val configuration = resources.configuration
    configuration.setLocale(locale)
    configuration.setLayoutDirection(locale)
    return context.createConfigurationContext(configuration)
  }

  fun changeLanguage(inputLocaleString: String?, activity: Activity?): Boolean {
    // no change
    if ((inputLocaleString.isNullOrBlank() && localeString.isNullOrBlank())
      || localeString == inputLocaleString
    ) {
      return true
    }

    // clear local if empty input locale
    if (inputLocaleString.isNullOrBlank()) {
      localeString = null
      updateSpLanguage(inputLocaleString)
      restartActivity(activity)
      return true
    }

    // invalid input locale
    if (!isValidLocale(inputLocaleString)) {
      return false
    }

    // update locale
    localeString = inputLocaleString
    updateSpLanguage(inputLocaleString)
    restartActivity(activity)
    return true
  }

  private fun updateSpLanguage(l: String?) {
    if (l.isNullOrBlank()) {
      sharedPreferences.edit().apply {
        remove(LOCALE_KEY)
      }.apply()
    } else {
      sharedPreferences.edit().apply {
        putString(LOCALE_KEY, l)
      }.apply()
    }
  }

  private fun restartActivity(activity: Activity?) {
    if (activity is FragmentActivity) {
      val fragments = activity.supportFragmentManager.fragments
      for (f in fragments) {
        if (f is FWBaseFragment && f.removeWhenRecreateActivity()) {
          FWFragmentUtil.removeFragment(activity, f)
        }
      }
    }
    activity?.recreate()
  }

  private fun isValidLocale(locale: String?) : Boolean {
    locale ?: return false
    val list = Locale.getAvailableLocales()
    if (locale.contains("-")) {
      val locales = locale.split("-")
      if (locales.size > 1) {
        return list.contains(Locale(locales[0], locales[1]))
      }
    }
    return list.contains(Locale(locale))
  }

  companion object : FWSingletonHolder<FWLanguageUtil, Context>(::FWLanguageUtil) {
    private const val LOCALE_SP = "fw_locale_sp"
    private const val LOCALE_KEY = "fw_locale_key"
  }
}
