//
//  FireworkSDKPlugin.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2022/2/21.
//

import FireworkVideoUI
import FireworkVideo
import Flutter
import Foundation

class FireworkSDKPlugin: NSObject, FlutterPlugin, FireworkVideoSDKDelegate {
    static var shareBaseURL: String?
    static var adBadgeConfiguration: AdBadgeConfiguration?
    static var videoLaunchBehavior: FWFVideoLaunchBehavior?
    private static var hasRegistered = false
    static let shared = FireworkSDKPlugin()

    var customCTAClickEnabled: Bool = false

    /// Decide whether to send video playback event to Dart.
    var enableVideoPlayBackEvent: Bool = false

    // player view controller
    var playerMap: NSMapTable<NSString, UIViewController> = NSMapTable.strongToWeakObjects()

    var channel: FlutterMethodChannel?

    private var hasCalledSDKInitFromNative: Bool = false
    private var argumentsForInitEvent: [String: Any]?

    static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(
            name: ChannelName.fireworkSDKModule.rawValue, binaryMessenger: registrar.messenger())
        let instance = hasRegistered ? FireworkSDKPlugin() : shared
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
        hasRegistered = true
    }

    override init() {
        super.init()
        _ = AppLanguageManager.shared
        NotificationCenter.default.addObserver(
            self, selector: #selector(Self.onAppLanguageChanged),
            name: AppLanguageManager.NotificationName.AppLanguageChanged, object: nil)
        DispatchQueue.once {
            UIViewController.swizzleLifecycleMethods()
        }
    }

    // swiftlint:disable cyclomatic_complexity
    func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case FireworkSDKPluginMethodName.`init`.rawValue:
            let sdkInitOptions = Convert.sdkInitOptions(call.arguments as? [String: AnyObject])
            initializeSDK(sdkInitOptions, false)
            result(nil)
        case FireworkSDKPluginMethodName.openVideoPlayer.rawValue:
            openVideoPlayer(call.arguments as? [String: AnyObject])
            result(nil)
        case FireworkSDKPluginMethodName.setCustomCTAClickEnabled.rawValue:
            setCustomCTAClickEnabled(call.arguments as? Bool)
            result(nil)
        case FireworkSDKPluginMethodName.setVideoPlaybackEventEnabled.rawValue:
            setVideoPlaybackEventEnabled(call.arguments as? Bool)
            result(nil)
        case FireworkSDKPluginMethodName.setShareBaseURL.rawValue:
            setShareBaseURL(call.arguments as? String)
            result(nil)
        case FireworkSDKPluginMethodName.setAdBadgeConfiguration.rawValue:
            setAdBadgeConfiguration(call.arguments as? [String: AnyObject])
            result(nil)
        case FireworkSDKPluginMethodName.trackPurchase.rawValue:
            trackPurchase(call.arguments as? [String: AnyObject])
            result(nil)
        case FireworkSDKPluginMethodName.changeAppLanguage.rawValue:
            changeAppLanguage(call.arguments as? String, result: result)
        case FireworkSDKPluginMethodName.pausePlayer.rawValue:
            pausePlayer(call.arguments as? String ?? "")
            result(nil)
        case FireworkSDKPluginMethodName.resumePlayer.rawValue:
            resumePlayer(call.arguments as? String ?? "")
            result(nil)
        case FireworkSDKPluginMethodName.markInitCalled.rawValue:
            if hasCalledSDKInitFromNative,
               let argumentsForInitEvent = argumentsForInitEvent {
                // The sdk init event was received before.
                channel?.invokeMethod(FWEventName.sdkInit.rawValue, arguments: argumentsForInitEvent)
            }
            result(hasCalledSDKInitFromNative)
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    // swiftlint:enable cyclomatic_complexity

    func initializeSDK(_ options: SDKInitOptions?, _ fromNative: Bool) {
        if hasCalledSDKInitFromNative && !fromNative {
            if let argumentsForInitEvent = argumentsForInitEvent {
                // The sdk init event was received before.
                channel?.invokeMethod(FWEventName.sdkInit.rawValue, arguments: argumentsForInitEvent)
            }
        } else {
            FireworkVideoSDK.initializeSDK(delegate: self, userID: options?.userId)
            FireworkVideoSDK.ctaDelegate = self
            FireworkVideoSDK.eventTracking.videoPlaybackDelegate = self
            FireworkVideoSDK.eventTracking.feedDelegate = self

            Self.videoLaunchBehavior = options?.videoLaunchBehavior
        }

        if fromNative {
            hasCalledSDKInitFromNative = true
        }
    }

    private func openVideoPlayer(_ config: [String: AnyObject]?) {
        guard let presentedVC = UIViewController.presentedViewController() else {
            return
        }

        guard let urlString = config?["url"] as? String, let url = URL.init(string: urlString) else {
            return
        }
        let playerViewConfigJson = (config?["config"] as? [String: AnyObject]) ?? [String: AnyObject]()
        let playerViewConfig = JsonHelper.fromDictionary(playerViewConfigJson, type: VideoPlayerConfiguration.self)

        let vpcConfig = VideoFeed.convertToVideoPlayerContentConfiguration(playerViewConfig)

        var feedContentConfig = VideoFeedContentConfiguration()
        if let vpcConfig = vpcConfig {
            feedContentConfig.playerView = vpcConfig
        }
        if let adBadge = Self.convertToFWAdBadgeConfiguration(Self.adBadgeConfiguration) {
            feedContentConfig.adBadge = adBadge
        }

        VideoFeedViewController.openVideoPlayer(
            with: url,
            feedContentConfig,
            presentedVC,
            playerViewConfig?.enablePictureInPicture ?? false
        ) { _ in

        }
    }

    private func setCustomCTAClickEnabled(_ enabled: Bool?) {
        if let enabled = enabled {
            customCTAClickEnabled = enabled
        }
    }

    private func setVideoPlaybackEventEnabled(_ enabled: Bool?) {
        if let enabled = enabled {
            enableVideoPlayBackEvent = enabled
        }
    }

    func setShareBaseURL(_ url: String?) {
        Self.shareBaseURL = url
    }

    func setAdBadgeConfiguration(_ config: [String: AnyObject]?) {
        guard let adBadgeConfiguration =
                JsonHelper.fromDictionary(config, type: AdBadgeConfiguration.self) else {
            return
        }

        Self.adBadgeConfiguration = adBadgeConfiguration
    }

    func trackPurchase(_ config: [String: AnyObject]?) {
        guard
            let trackPurchaseParameters = JsonHelper.fromDictionary(
                config, type: TrackPurchaseParameters.self)
        else {
            return
        }

        FireworkVideoSDK.trackPurchase(
            orderID: trackPurchaseParameters.orderId,
            value: trackPurchaseParameters.value,
            currencyCode: trackPurchaseParameters.currencyCode,
            countryCode: trackPurchaseParameters.countryCode,
            trackPurchaseParameters.additionalInfo
        )
    }

    func changeAppLanguage(_ language: String?, result: @escaping FlutterResult) {
        let changeAppLanguageResult = AppLanguageManager.shared.changeAppLanguage(language)
        result(changeAppLanguageResult)
    }

    // MARK: - FireworkVideoSDKDelegate
    func fireworkVideoDidLoadSuccessfully() {
        argumentsForInitEvent = [:]

        channel?.invokeMethod(FWEventName.sdkInit.rawValue, arguments: argumentsForInitEvent)
    }

    func fireworkVideoDidLoadWith(error: FireworkVideoSDKError) {
        argumentsForInitEvent = [
            "error": [
                "name": error.jsErrorName, "reason": error.recoverySuggestion ?? "default"
            ]
        ]

        channel?.invokeMethod(
            FWEventName.sdkInit.rawValue,
            arguments: argumentsForInitEvent)
    }

    @objc func onAppLanguageChanged() {
        channel?.invokeMethod(
            FWEventName.nativeAppLanguageUpdated.rawValue,
            arguments: [
                "appLanguage": AppLanguageManager.shared.appLanguage ?? ""
            ])
    }
}

extension FireworkSDKPlugin {
    static func convertToFWAdBadgeConfiguration(_ adBadgeConfiguration: AdBadgeConfiguration?)
        -> FireworkVideo.AdBadgeConfiguration? {
        guard let adBadgeConfiguration = adBadgeConfiguration else {
            return nil
        }

        var fwAdBadgeConfiguration = FireworkVideo.AdBadgeConfiguration()
            if let textColor = adBadgeConfiguration.textColor, let color = textColor.uicolor() {
            fwAdBadgeConfiguration.textColor = color
        }
            if let backgroundColor = adBadgeConfiguration.backgroundColor, let color = backgroundColor.uicolor() {
            fwAdBadgeConfiguration.backgroundColor = color
        }
        switch adBadgeConfiguration.badgeTextType {
        case .sponsored:
            fwAdBadgeConfiguration.badgeText =
                FireworkVideo.AdBadgeConfiguration.BadgeText.sponsored
        default:
            fwAdBadgeConfiguration.badgeText = FireworkVideo.AdBadgeConfiguration.BadgeText.ad
        }

        return fwAdBadgeConfiguration
    }
}
