import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import '../../models/method_names.dart';
import '../../models/story_block_configuration.dart';
import '../../utils/fw_method_channel_util.dart';
import '../../utils/model_utils/fw_error_util.dart';
import '../../utils/model_utils/method_names_util.dart';
import '../../utils/model_utils/story_block_configuration_util.dart';
import 'story_block_native_callbacks.dart';

class StoryBlockPlatformHandlerCallbacks {
  final StoryBlockPlatformViewLoadFinishedCallback? onPlatformViewLoadFinished;
  final StoryBlockPlatformViewEmptyCallback? onPlatformViewEmpty;
  final StoryBlockPlatformViewPdpVisibilityChangeCallback?
      onPlatformViewPdpVisibilityChange;
  final StoryBlockPlatformViewDidStartPictureInPictureCallback?
      onPlatformViewDidStartPictureInPicture;
  final StoryBlockPlatformViewDidStopPictureInPictureCallback?
      onPlatformViewDidStopPictureInPicture;
  final StoryBlockPlatformViewOnGetFeedIdCallback? onPlatformViewGetFeedId;

  StoryBlockPlatformHandlerCallbacks({
    this.onPlatformViewLoadFinished,
    this.onPlatformViewEmpty,
    this.onPlatformViewPdpVisibilityChange,
    this.onPlatformViewDidStartPictureInPicture,
    this.onPlatformViewDidStopPictureInPicture,
    this.onPlatformViewGetFeedId,
  });
}

class StoryBlockPlatformHandler {
  final MethodChannel _channel;
  final StoryBlockPlatformHandlerCallbacks _callbacks;
  StoryBlockPlatformHandler({
    required int viewId,
    required callbacks,
  })  : _channel = FWMethodChannelUtil.getStoryBlockChannel(viewId),
        _callbacks = callbacks {
    _channel.setMethodCallHandler(_handleMethodCall);
  }

  void updateStoryBlockConfiguration(
      StoryBlockConfiguration? storyBlockConfiguration) {
    final newStoryBlockConfigurationJson =
        StoryBlockConfigurationUtil.toJson(storyBlockConfiguration);
    _setStoryBlockConfiguration(newStoryBlockConfigurationJson);
  }

  Future<void> play() async {
    final nativeMethod = StoryBlockNativeMethodName.play.name;
    await _channel.invokeMethod(nativeMethod);
  }

  Future<void> pause() async {
    final nativeMethod = StoryBlockNativeMethodName.pause.name;
    await _channel.invokeMethod(nativeMethod);
  }

  Future<void> setMarginRight(double marginRight) async {
    if (defaultTargetPlatform == TargetPlatform.android) {
      final nativeMethod = StoryBlockNativeMethodName.setMarginRight.name;
      await _channel.invokeMethod(
        nativeMethod,
        marginRight,
      );
    }
  }

  Future<void> showIvsPlayerView() async {
    if (defaultTargetPlatform == TargetPlatform.android) {
      final nativeMethod = StoryBlockNativeMethodName.showIvsPlayerView.name;
      await _channel.invokeMethod(
        nativeMethod,
      );
    }
  }

  Future<void> hideIvsPlayerView() async {
    if (defaultTargetPlatform == TargetPlatform.android) {
      final nativeMethod = StoryBlockNativeMethodName.hideIvsPlayerView.name;
      await _channel.invokeMethod(
        nativeMethod,
      );
    }
  }

  Future<void> removeIvsPlayerView() async {
    if (defaultTargetPlatform == TargetPlatform.android) {
      final nativeMethod = StoryBlockNativeMethodName.removeIvsPlayerView.name;
      await _channel.invokeMethod(
        nativeMethod,
      );
    }
  }

  Future<void> playIvsPlayer() async {
    if (defaultTargetPlatform == TargetPlatform.android) {
      final nativeMethod = StoryBlockNativeMethodName.playIvsPlayer.name;
      await _channel.invokeMethod(
        nativeMethod,
      );
    }
  }

  Future<void> pauseIvsPlayer() async {
    if (defaultTargetPlatform == TargetPlatform.android) {
      final nativeMethod = StoryBlockNativeMethodName.pauseIvsPlayer.name;
      await _channel.invokeMethod(
        nativeMethod,
      );
    }
  }

  Future<void> onViewportEntered() async {
    final nativeMethod = StoryBlockNativeMethodName.onViewportEntered.name;
    await _channel.invokeMethod(
      nativeMethod,
    );
  }

  Future<void> onViewportLeft() async {
    final nativeMethod = StoryBlockNativeMethodName.onViewportLeft.name;
    await _channel.invokeMethod(
      nativeMethod,
    );
  }

  Future<void> openFullscreen() async {
    if (defaultTargetPlatform == TargetPlatform.android) {
      final nativeMethod = StoryBlockNativeMethodName.openFullscreen.name;
      await _channel.invokeMethod(
        nativeMethod,
      );
    }
  }

  Future<dynamic> _handleMethodCall(MethodCall call) async {
    final methodName = StoryBlockMethodNameUtil.createMethodName(call.method);
    if (methodName == null) {
      return null;
    }

    switch (methodName) {
      case StoryBlockMethodName.onStoryBlockLoadFinished:
        final error = FWErrorUtil.fromJson(call.arguments);
        _callbacks.onPlatformViewLoadFinished?.call(error);
        break;
      case StoryBlockMethodName.onStoryBlockEmpty:
        _callbacks.onPlatformViewEmpty?.call(null);
        break;
      case StoryBlockMethodName.onStoryBlockPdpVisibilityChange:
        bool visible = false;
        if (call.arguments is Map<dynamic, dynamic>) {
          final arg = call.arguments as Map<dynamic, dynamic>;
          if (arg["visible"] is bool) {
            visible = arg["visible"] as bool;
          }
        }
        _callbacks.onPlatformViewPdpVisibilityChange?.call(visible);
        break;
      case StoryBlockMethodName.onStoryBlockDidStartPictureInPicture:
        final error = FWErrorUtil.fromJson(call.arguments);
        _callbacks.onPlatformViewDidStartPictureInPicture?.call(error);
        break;
      case StoryBlockMethodName.onStoryBlockDidStopPictureInPicture:
        final error = FWErrorUtil.fromJson(call.arguments);
        _callbacks.onPlatformViewDidStopPictureInPicture?.call(error);
        break;
      case StoryBlockMethodName.onStoryBlockGetFeedId:
        String feedId = "";
        if (call.arguments is Map<dynamic, dynamic>) {
          final arg = call.arguments as Map<dynamic, dynamic>;
          if (arg["feedId"] is String) {
            feedId = arg["feedId"] as String;
          }
        }
        _callbacks.onPlatformViewGetFeedId?.call(feedId);
        break;
    }

    return null;
  }

  void _setStoryBlockConfiguration(Map<String, dynamic> storyBlockConfigJson) {
    final nativeMethod =
        StoryBlockNativeMethodName.setStoryBlockConfiguration.name;

    _channel.invokeMethod(
      nativeMethod,
      storyBlockConfigJson,
    );
  }
}
