import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

import '../../constants/fw_event_name.dart';
import '../../firework_sdk.dart';
import '../../fw_event_bus.dart';
import '../../models/ad_configuration.dart';
import '../../models/button_info.dart';
import '../../models/fw_error.dart';
import '../../models/fw_native_error.dart';
import '../../models/video_feed_configuration.dart';
import '../../models/video_feed_mode.dart';
import '../../models/video_feed_source.dart';
import '../../models/video_player_configuration.dart';
import '../../models/video_player_native_configuration.dart';
import '../../utils/fw_global_state.dart';
import '../../utils/fw_json_util.dart';
import '../../utils/model_utils/countdown_timer_configuration_util.dart';
import '../../utils/model_utils/replay_badge_configuration_util.dart';
import '../../utils/model_utils/video_player_logo_configuration_util.dart';
import 'video_feed_controller.dart';
import 'video_feed_platform_handler.dart';
import 'video_feed_platform_view.dart';

typedef VideoFeedCreatedCallback = void Function(
    VideoFeedController controller);
typedef VideoFeedLoadFinishedCallback = void Function(FWError? error);
typedef VideoFeedEmptyCallback = void Function(FWError? error);
typedef VideoFeedDidStartPictureInPictureCallback = void Function(
    FWError? error);
typedef VideoFeedDidStopPictureInPictureCallback = void Function(
    FWError? error);
typedef VideoFeedGetFeedIdCallback = void Function(String feedId);

/// The widget of video feed.
class VideoFeed extends StatefulWidget {
  /// The source of the feed.
  /// The playlistGroup is only supported on iOS.
  /// Defaults to discover.
  final VideoFeedSource source;

  /// The channel id of the feed.
  /// Required when the source is set as `channel` or `playlist` or `dynamicContent`.
  final String? channel;

  /// The playlist id of the feed.
  /// Required when the source is set as `playlist`.
  final String? playlist;

  /// Playlist group id of the feed.
  /// Required when the source is set as `playlistGroup`.
  /// Only supported on iOS.
  final String? playlistGroup;

  /// The dynamic content parameters of the feed.
  /// Required when the source is set as `dynamicContent`.
  final Map<String, List<String>>? dynamicContentParameters;

  /// Hashtag filter expression is an s-expression used to provide feeds filtered by hashtags with specified criteria.
  /// Queries are specified with boolean predicates on what hashtags are there on the video.
  /// For instance, (and sport food) (or sport food) (and sport (or food comedy)) sport are all valid expressions.
  /// Non-UTF-8 characters are not allowed. If using boolean predicates, the expression needs to be wrapped with parenthesis.
  /// Required when the source is set as hashtagPlaylist.
  final String? hashtagFilterExpression;

  /// Product ids used to generate the sku feed.
  /// Required when the source is set as sku.
  final List<String>? productIds;

  /// The video or live stream id.
  final String? contentId;

  /// One of three available display modes.
  /// Defaults to row.
  final VideoFeedMode? mode;

  /// Specifies if Picture in Picture is enabled.
  final bool? enablePictureInPicture;

  /// The width of the feed.
  final double? width;

  /// The height of the feed.
  final double? height;

  /// Set this property to true if you want keep the widget alive during scrolling list out of view box.
  /// Default to false.
  final bool? wantKeepAlive;

  /// The configuration of the feed.
  final VideoFeedConfiguration? videoFeedConfiguration;

  /// The configuration of the player.
  final VideoPlayerConfiguration? videoPlayerConfiguration;

  /// Ad configuration of the feed.
  final AdConfiguration? adConfiguration;

  /// The callback of video feed creation.
  final VideoFeedCreatedCallback? onVideoFeedCreated;

  /// The callback of the feed loading result.
  /// It means loading successfully when error equals to null.
  final VideoFeedLoadFinishedCallback? onVideoFeedLoadFinished;

  /// The callback is triggered when there are no items in the video feed.
  /// The callback is triggered in the following cases:
  ///  1. Loading successfully but the back end returns an empty list.
  ///  2. The load failed and list is empty.
  /// onVideoFeedLoadFinished will also be triggered when onVideoFeedEmpty is triggered.
  final VideoFeedEmptyCallback? onVideoFeedEmpty;

  /// The callback of the feed starting picture in picture.
  /// It means starting successfully when error equals to null.
  /// Only supported on iOS.
  final VideoFeedDidStartPictureInPictureCallback?
      onVideoFeedDidStartPictureInPicture;

  /// The callback of the feed stopping picture in picture.
  /// It means stopping successfully when error equals to null.
  /// Only supported on iOS.
  final VideoFeedDidStopPictureInPictureCallback?
      onVideoFeedDidStopPictureInPicture;

  /// The host app could use this callback to get feed id.
  /// The feed id can be used for conversion tracking.
  final VideoFeedGetFeedIdCallback? onVideoFeedGetFeedId;

  const VideoFeed({
    Key? key,
    required this.source,
    this.channel,
    this.playlist,
    this.playlistGroup,
    this.dynamicContentParameters,
    this.hashtagFilterExpression,
    this.productIds,
    this.contentId,
    this.mode,
    this.enablePictureInPicture,
    this.width,
    this.height,
    this.wantKeepAlive,
    this.videoFeedConfiguration,
    this.videoPlayerConfiguration,
    this.adConfiguration,
    this.onVideoFeedCreated,
    this.onVideoFeedLoadFinished,
    this.onVideoFeedEmpty,
    this.onVideoFeedDidStartPictureInPicture,
    this.onVideoFeedDidStopPictureInPicture,
    this.onVideoFeedGetFeedId,
  }) : super(key: key);

  @override
  State<VideoFeed> createState() => _VideoFeedState();
}

class _VideoFeedState extends State<VideoFeed>
    with AutomaticKeepAliveClientMixin {
  VideoFeedController? _controller;
  StreamSubscription? _eventSubscription;
  bool _sdkInitCalled = false;

  @override
  void initState() {
    super.initState();
    _eventSubscription = FWEventBus.getInstance().on().listen((event) {
      if (mounted) {
        if (event.eventName == FWEventName.adBadgeConfigurationUpdated) {
          setState(() {});
        } else if (event.eventName == FWEventName.shareBaseURLUpdated) {
          setState(() {});
        } else if (event.eventName == FWEventName.videoLaunchBehaviorUpdated) {
          setState(() {});
        } else if (event.eventName == FWEventName.appLanguageUpdated) {
          setState(() {});
        }
      }
    });

    _sdkInitCalled = FWGlobalState.getInstance().sdkInitCalled;
    if (!_sdkInitCalled) {
      WidgetsBinding.instance.addPostFrameCallback((_) {
        FWGlobalState.getInstance().sdkInitCalledFuture.then((__) {
          setState(() {
            _sdkInitCalled = true;
          });
        });
      });
    }
  }

  @override
  void didUpdateWidget(covariant VideoFeed oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.wantKeepAlive != widget.wantKeepAlive) {
      updateKeepAlive();
    }
  }

  @override
  void dispose() {
    _eventSubscription?.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);

    if (!_sdkInitCalled) {
      return SizedBox(
        width: widget.width,
        height: widget.height,
      );
    }

    return SizedBox(
      width: widget.width,
      height: widget.height,
      child: VideoFeedPlatformView(
        key: ValueKey(_generateKey()),
        source: widget.source,
        channel: widget.channel,
        playlist: widget.playlist,
        playlistGroup: widget.playlistGroup,
        dynamicContentParameters: widget.dynamicContentParameters,
        hashtagFilterExpression: widget.hashtagFilterExpression,
        productIds: widget.productIds,
        contentId: widget.contentId,
        mode: widget.mode ?? VideoFeedMode.row,
        videoFeedConfiguration: widget.videoFeedConfiguration?.deepCopy(),
        videoPlayerConfiguration:
            VideoPlayerNativeConfiguration.withPlayerConfiguration(
          playerConfiguration: widget.videoPlayerConfiguration?.deepCopy(),
          enablePictureInPicture: widget.enablePictureInPicture,
        ),
        adConfiguration: widget.adConfiguration?.deepCopy(),
        onVideoFeedPlatformViewCreated: _onVideoFeedPlatformViewCreated,
        onVideoFeedPlatformViewLoadFinished:
            _onVideoFeedPlatformViewLoadFinished,
        onVideoFeedPlatformViewEmpty: widget.onVideoFeedEmpty,
        onVideoFeedPlatformViewDidStartPictureInPicture:
            widget.onVideoFeedDidStartPictureInPicture,
        onVideoFeedPlatformViewDidStopPictureInPicture:
            widget.onVideoFeedDidStopPictureInPicture,
        onVideoFeedPlatformViewGetFeedId: widget.onVideoFeedGetFeedId,
      ),
    );
  }

  String _generateKey() {
    final gShareBaseURL = FireworkSDK.getInstance().shareBaseURL;
    final appLanguage = FireworkSDK.getInstance().appLanguage;
    final videoLaunchBehavior =
        FireworkSDK.getInstance().videoLaunchBehavior?.name;
    final adBadgeConfiguration = FireworkSDK.getInstance().adBadgeConfiguration;
    final adBadgeTextType = adBadgeConfiguration?.badgeTextType?.name;
    final backgroundColorOfAdBadge = adBadgeConfiguration?.backgroundColor;
    final textColorOfAdBadge = adBadgeConfiguration?.textColor;
    final androidFontIsCustomOfAdBadge =
        adBadgeConfiguration?.androidFontInfo?.isCustom;
    final androidFontTypefaceNameOfAdBadge =
        adBadgeConfiguration?.androidFontInfo?.typefaceName;

    final source = widget.source.name;
    final channel = widget.channel;
    final playlist = widget.playlist;
    final playlistGroup = widget.playlistGroup;
    final dynamicContentParametersString =
        _generateDynamicContentParametersString();
    final hashtagFilterExpression = widget.hashtagFilterExpression;
    final productIds = widget.productIds?.join(',');
    final contentId = widget.contentId;
    final mode = widget.mode?.name;
    final enablePictureInPicture = widget.enablePictureInPicture;

    final backgroundColor = widget.videoFeedConfiguration?.backgroundColor;
    final cornerRadius =
        widget.videoFeedConfiguration?.cornerRadius?.toStringAsFixed(5);
    final titleHidden = widget.videoFeedConfiguration?.title?.hidden;
    final titleTextColor = widget.videoFeedConfiguration?.title?.textColor;
    final titleFontSize =
        widget.videoFeedConfiguration?.title?.fontSize?.toStringAsFixed(0);
    final titleAndroidFontInfo =
        widget.videoFeedConfiguration?.title?.androidFontInfo;
    final titleGradientDrawable =
        widget.videoFeedConfiguration?.title?.gradientDrawable;
    final titlePosition = widget.videoFeedConfiguration?.titlePosition?.name;
    final playIconHidden = widget.videoFeedConfiguration?.playIcon?.hidden;
    final playIconWidth =
        widget.videoFeedConfiguration?.playIcon?.iconWidth?.toStringAsFixed(0);
    final showAdBadge = widget.videoFeedConfiguration?.showAdBadge;
    final enableAutoplay = widget.videoFeedConfiguration?.enableAutoplay;
    final gridColumns = widget.videoFeedConfiguration?.gridColumns;
    final itemSpacing =
        widget.videoFeedConfiguration?.itemSpacing?.toStringAsFixed(5);
    final replayBadgeForFeed = widget.videoFeedConfiguration?.replayBadge;

    final playerStyle = widget.videoPlayerConfiguration?.playerStyle?.name;
    final videoCompleteAction =
        widget.videoPlayerConfiguration?.videoCompleteAction?.name;
    final showShareButton = widget.videoPlayerConfiguration?.showShareButton;
    final showPlaybackButton =
        widget.videoPlayerConfiguration?.showPlaybackButton;
    final showMuteButton = widget.videoPlayerConfiguration?.showMuteButton;
    final showBranding = widget.videoPlayerConfiguration?.showBranding;
    final ctaDelay = widget.videoPlayerConfiguration?.ctaDelay;
    final ctaDelayType = ctaDelay?.type.name;
    final ctaDelayValue = ctaDelay?.value.toStringAsFixed(5);
    final ctaHighlightDelay =
        widget.videoPlayerConfiguration?.ctaHighlightDelay;
    final ctaHighlightDelayType = ctaHighlightDelay?.type.name;
    final ctaHighlightDelayValue = ctaHighlightDelay?.value.toStringAsFixed(5);
    final shareBaseURL = widget.videoPlayerConfiguration?.shareBaseURL;
    final ctaWidth = widget.videoPlayerConfiguration?.ctaWidth?.name;
    final buttonConfiguration =
        widget.videoPlayerConfiguration?.buttonConfiguration;
    final videoPlayerLogoConfiguration =
        widget.videoPlayerConfiguration?.videoPlayerLogoConfiguration;
    final showVideoDetailTitle =
        widget.videoPlayerConfiguration?.showVideoDetailTitle;
    final replayBadgeForPlayer =
        widget.videoPlayerConfiguration?.replayBadgeConfiguration;
    final countdownTimerConfiguration =
        widget.videoPlayerConfiguration?.countdownTimerConfiguration;
    final requiresAds = widget.adConfiguration?.requiresAds;
    final adsFetchTimeout = widget.adConfiguration?.adsFetchTimeout;
    final vastAttributesString = _generateVastAttributesString();

    String key = "_gShareBaseURL:$gShareBaseURL";
    if (defaultTargetPlatform == TargetPlatform.iOS) {
      key += "_appLanguage:$appLanguage";
    }
    key += "_videoLaunchBehavior:$videoLaunchBehavior";
    key += "_adBadgeTextType:$adBadgeTextType";
    key += "_backgroundColorOfAdBadge:$backgroundColorOfAdBadge";
    key += "_textColorOfAdBadge:$textColorOfAdBadge";
    if (defaultTargetPlatform == TargetPlatform.android) {
      key += "_androidFontIsCustomOfAdBadge:$androidFontIsCustomOfAdBadge";
      key +=
          "_androidFontTypefaceNameOfAdBadge:$androidFontTypefaceNameOfAdBadge";
    }

    key += "_source:$source";
    key += "_channel:$channel";
    key += "_playlist:$playlist";
    key += "_playlistGroup:$playlistGroup";
    key += "_dynamicContentParameters:$dynamicContentParametersString";
    key += "_hashtagFilterExpression:$hashtagFilterExpression";
    key += "_productIds:$productIds";
    key += "_contentId:$contentId";
    key += "_mode:$mode";
    key += "_enablePictureInPicture:$enablePictureInPicture";

    key += "_titleHidden:$titleHidden";
    if (defaultTargetPlatform == TargetPlatform.android) {
      key += "_backgroundColor:$backgroundColor";
      key += "_cornerRadius:$cornerRadius";
      key += "_titleTextColor:$titleTextColor";
      key += "_titleFontSize:$titleFontSize";
      key += "_titleAndroidFontInfo.isCustom:${titleAndroidFontInfo?.isCustom}";
      key +=
          "_titleAndroidFontInfo.typefaceName:${titleAndroidFontInfo?.typefaceName}";
      key +=
          "_titleGradientDrawable.orientation:${titleGradientDrawable?.orientation?.rawName}";
      key +=
          "_titleGradientDrawable.colors:${titleGradientDrawable?.colors?.join(",")}";
      key += "_playIconHidden:$playIconHidden";
      key += "_playIconWidth:$playIconWidth";
      key += "_showAdBadge:$showAdBadge";
      key += "_itemSpacing:$itemSpacing";
    }
    key += "_titlePosition:$titlePosition";
    if (defaultTargetPlatform == TargetPlatform.android) {
      key += "_enableAutoplay:$enableAutoplay";
    }
    key += "_gridColumns:$gridColumns";
    if (replayBadgeForFeed != null) {
      final replayBadgeForFeedJson =
          ReplayBadgeConfigurationUtil.toJson(replayBadgeForFeed);
      key +=
          "_replayBadgeForFeed:${FWJsonUtil.generateJsonKey(replayBadgeForFeedJson)}";
    }

    key += "_shareBaseURL:$shareBaseURL";
    if (defaultTargetPlatform == TargetPlatform.android) {
      key += "_playerStyle:$playerStyle";
      key += "_videoCompleteAction:$videoCompleteAction";
      key += "_showShareButton:$showShareButton";
      key += "_showPlaybackButton:$showPlaybackButton";
      key += "_showMuteButton:$showMuteButton";
      key += "_showBranding:$showBranding";
      key += "_ctaDelayType:$ctaDelayType";
      key += "_ctaDelayValue:$ctaDelayValue";
      key += "_ctaHighlightDelayType:$ctaHighlightDelayType";
      key += "_ctaHighlightDelayValue:$ctaHighlightDelayValue";
      key += "_ctaWidth:$ctaWidth";
      key +=
          "_buttonConfiguration.videoDetailButton:${_generateButtonInfoString(buttonConfiguration?.videoDetailButton)}";
      key +=
          "_buttonConfiguration.closeButton:${_generateButtonInfoString(buttonConfiguration?.closeButton)}";
      key +=
          "_buttonConfiguration.muteButton:${_generateButtonInfoString(buttonConfiguration?.muteButton)}";
      key +=
          "_buttonConfiguration.unmuteButton:${_generateButtonInfoString(buttonConfiguration?.unmuteButton)}";
      key +=
          "_buttonConfiguration.playButton:${_generateButtonInfoString(buttonConfiguration?.playButton)}";
      key +=
          "_buttonConfiguration.pauseButton:${_generateButtonInfoString(buttonConfiguration?.pauseButton)}";
      if (videoPlayerLogoConfiguration != null) {
        final videoPlayerLogoConfigurationJson =
            VideoPlayerLogoConfigurationUtil.toJson(
                videoPlayerLogoConfiguration);
        key +=
            "_videoPlayerLogoConfiguration:${FWJsonUtil.generateJsonKey(videoPlayerLogoConfigurationJson)}";
      }
      key += "_showVideoDetailTitle:$showVideoDetailTitle";
      if (countdownTimerConfiguration != null) {
        final countdownTimerConfigurationJson =
            CountdownTimerConfigurationUtil.toJson(countdownTimerConfiguration);
        key +=
            "_countdownTimerConfiguration:${FWJsonUtil.generateJsonKey(countdownTimerConfigurationJson)}";
      }
    }

    if (replayBadgeForPlayer != null) {
      final replayBadgeForPlayerJson =
          ReplayBadgeConfigurationUtil.toJson(replayBadgeForPlayer);
      key +=
          "_replayBadgeForPlayer:${FWJsonUtil.generateJsonKey(replayBadgeForPlayerJson)}";
    }

    key += "_requiresAds:$requiresAds";
    key += "_adsFetchTimeout:$adsFetchTimeout";
    key += "_vastAttributesString:$vastAttributesString";

    return key;
  }

  String _generateDynamicContentParametersString() {
    final parameters = widget.dynamicContentParameters;
    if (parameters == null) {
      return "";
    }

    var resultString = "";
    final sortedKeyList = parameters.keys.toList()..sort();
    for (final key in sortedKeyList) {
      final value = parameters[key];
      if (value != null) {
        final valueString = value.join(",");
        if (resultString.isNotEmpty) {
          resultString += "_";
        }
        resultString += "$key:$valueString";
      }
    }

    return resultString;
  }

  String _generateVastAttributesString() {
    final vastAttributes = widget.adConfiguration?.vastAttributes;
    if (vastAttributes == null) {
      return "";
    }

    var resultString = "";
    for (var attribute in vastAttributes) {
      final name = attribute.name;
      final value = attribute.value;
      if (resultString.isNotEmpty) {
        resultString += "_";
      }
      resultString += "$name:$value";
    }
    return resultString;
  }

  String _generateButtonInfoString(ButtonInfo? buttonInfo) {
    return "imageName:${buttonInfo?.imageName}_systemImageName:${buttonInfo?.systemImageName}_tintColor:${buttonInfo?.tintColor}";
  }

  void _onVideoFeedPlatformViewCreated(VideoFeedPlatformHandler handler) {
    if (_controller == null) {
      _controller = VideoFeedController(handler: handler);
      widget.onVideoFeedCreated?.call(_controller!);
    } else {
      _controller!.handler = handler;
    }
  }

  void _onVideoFeedPlatformViewLoadFinished(FWNativeError? error) {
    widget.onVideoFeedLoadFinished?.call(error?.toError());
  }

  @override
  bool get wantKeepAlive => widget.wantKeepAlive ?? false;
}
