import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import '../../models/method_names.dart';
import '../../models/video_feed_configuration.dart';
import '../../models/video_player_configuration.dart';
import '../../utils/fw_method_channel_util.dart';
import '../../utils/model_utils/fw_error_util.dart';
import '../../utils/model_utils/method_names_util.dart';
import '../../utils/model_utils/video_feed_configuration_util.dart';
import '../../utils/model_utils/video_player_configuration_util.dart';
import 'video_feed_native_callbacks.dart';

class VideoFeedPlatformHandlerCallbacks {
  final VideoFeedPlatformViewLoadFinishedCallback? onPlatformViewLoadFinished;
  final VideoFeedPlatformViewEmptyCallback? onPlatformViewEmpty;
  final VideoFeedPlatformViewDidStartPictureInPictureCallback?
      onPlatformViewDidStartPictureInPicture;
  final VideoFeedPlatformViewDidStopPictureInPictureCallback?
      onPlatformViewDidStopPictureInPicture;
  final VideoFeedPlatformViewGetFeedIdCallback? onPlatformViewGetFeedId;
  VideoFeedPlatformHandlerCallbacks({
    this.onPlatformViewLoadFinished,
    this.onPlatformViewEmpty,
    this.onPlatformViewDidStartPictureInPicture,
    this.onPlatformViewDidStopPictureInPicture,
    this.onPlatformViewGetFeedId,
  });
}

class VideoFeedPlatformHandler {
  final MethodChannel _channel;
  final VideoFeedPlatformHandlerCallbacks _callbacks;

  VideoFeedPlatformHandler({
    required int viewId,
    required VideoFeedPlatformHandlerCallbacks callbacks,
  })  : _channel = FWMethodChannelUtil.getVideoFeedChannel(viewId),
        _callbacks = callbacks {
    _channel.setMethodCallHandler(_handleMethodCall);
  }

  void updateVideoFeedConfiguration(
      VideoFeedConfiguration? videoFeedConfiguration) {
    final newVideoFeedConfigurationJson =
        VideoFeedConfigurationUtil.toJson(videoFeedConfiguration);
    _setVideoFeedConfiguration(newVideoFeedConfigurationJson);
  }

  void updateVideoPlayerConfiguration(
      VideoPlayerConfiguration? videoPlayerConfiguration) {
    final newVideoPlayerConfigurationJson =
        VideoPlayerConfigurationUtil.toJson(videoPlayerConfiguration);
    _setVideoPlayerConfiguration(newVideoPlayerConfigurationJson);
  }

  void refresh() {
    final nativeMethod = VideoFeedNativeMethodName.refresh.name;

    _channel.invokeMethod(
      nativeMethod,
    );
  }

  Future<void> setMarginRight(double marginRight) async {
    if (defaultTargetPlatform == TargetPlatform.android) {
      final nativeMethod = VideoFeedNativeMethodName.setMarginRight.name;
      await _channel.invokeMethod(
        nativeMethod,
        marginRight,
      );
    }
  }

  Future<dynamic> _handleMethodCall(MethodCall call) async {
    final methodName = VideoFeedMethodNameUtil.createMethodName(call.method);
    if (methodName == null) {
      return null;
    }

    switch (methodName) {
      case VideoFeedMethodName.onVideoFeedLoadFinished:
        final error = FWErrorUtil.createNativeErrorfromJson(call.arguments);
        _callbacks.onPlatformViewLoadFinished?.call(error);
        break;
      case VideoFeedMethodName.onVideoFeedEmpty:
        _callbacks.onPlatformViewEmpty?.call(null);
        break;
      case VideoFeedMethodName.onVideoFeedDidStartPictureInPicture:
        final error = FWErrorUtil.fromJson(call.arguments);
        _callbacks.onPlatformViewDidStartPictureInPicture?.call(error);
        break;
      case VideoFeedMethodName.onVideoFeedDidStopPictureInPicture:
        final error = FWErrorUtil.fromJson(call.arguments);
        _callbacks.onPlatformViewDidStopPictureInPicture?.call(error);
        break;
      case VideoFeedMethodName.onVideoFeedGetFeedId:
        String feedId = "";
        if (call.arguments is Map<dynamic, dynamic>) {
          final arg = call.arguments as Map<dynamic, dynamic>;
          if (arg["feedId"] is String) {
            feedId = arg["feedId"] as String;
          }
        }
        _callbacks.onPlatformViewGetFeedId?.call(feedId);
        break;
    }

    return null;
  }

  void _setVideoFeedConfiguration(Map<String, dynamic> feedConfigJson) {
    final nativeMethod =
        VideoFeedNativeMethodName.setVideoFeedConfiguration.name;

    _channel.invokeMethod(
      nativeMethod,
      feedConfigJson,
    );
  }

  void _setVideoPlayerConfiguration(Map<String, dynamic> playerConfigJson) {
    final nativeMethod =
        VideoFeedNativeMethodName.setVideoPlayerConfiguration.name;

    _channel.invokeMethod(
      nativeMethod,
      playerConfigJson,
    );
  }
}
